from django.db import models
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User

import datetime

PACKAGE_GREEN_THRESHOLD = 5
PACKAGE_RED_THRESHOLD = 20

EXPERIENCE_CHOICES = (
    (0, '---'),
    (1, 'Easy'),
    (2, 'Medium'),
    (3, 'Hard'),
)

class PackageSet(models.Model):
    name = models.SlugField(_("Name"), max_length=40)

    def __unicode__(self):
        return self.name


class SourcePackage(models.Model):
    name = models.SlugField(_("Name"), max_length=70)
    packagesets = models.ManyToManyField(PackageSet, null=True)

    class Meta:
        ordering = ['name']

    def __unicode__(self):
        return self.name

    @property
    def valid_opportunities(self):
        return self.opportunity_set.filter(valid=True)

    @property
    def opportunity_class(self):
        if self.valid_opportunities.count() <= PACKAGE_GREEN_THRESHOLD:
            return 'GREEN'
        if self.valid_opportunities.count() >= PACKAGE_RED_THRESHOLD:
            return 'RED'
        else:
            return 'YELLOW'


class OpportunityList(models.Model):
    name = models.SlugField(_("Name"), max_length=70)
    url = models.URLField(_("URL"))
    description = models.TextField(_("Description"), max_length=350)
    last_updated = models.DateTimeField(_("Last Updated"), null=True)
    active = models.BooleanField(_("Active"), default=True)
    featured = models.BooleanField(_("Featured"), default=False, 
                                   help_text=_("Specially feature this list of opportunities?"))
    explanation = models.TextField(_("Explanatory help text"), max_length=500, null=True)
    commentable = models.BooleanField(_("Commentable"), default=True, 
                                      help_text=_("Can opportunities on this list have comments?"))
    experience = models.IntegerField(_("Required Experience"), default=0, choices=EXPERIENCE_CHOICES,
                                     help_text=_("Level of experience required for this type of opportunities."))

    class Meta:
        ordering = ['name']

    def __unicode__(self):
        return self.name

    @property
    def opportunity_urgency(self):
        if self.featured:
            return 'YES'
        return 'NO'


class Opportunity(models.Model):
    description = models.TextField(_("Description"), max_length=350)
    url = models.URLField(_("URL"), max_length=350)
    last_updated = models.DateTimeField(_("Last Updated"), null=True)
    since = models.DateTimeField(_("Since"), help_text=_("On the list since"), null=True)
    reviewed = models.BooleanField(_("Irrelevant"), default=False, blank=True)
    applied = models.BooleanField(_("Applied"), default=False, blank=True)
    sourcepackage = models.ForeignKey(SourcePackage)
    opportunitylist = models.ForeignKey(OpportunityList)
    comment = models.TextField(_("Comment"), blank=True)
    valid = models.BooleanField(_("Valid"), default=True)
    experience = models.IntegerField(_("Required Experience"), choices=EXPERIENCE_CHOICES, default=0,
                                     help_text=_("Level of experience required for this specific opportunity."))

    class Meta:
        ordering = ['-last_updated']
        verbose_name_plural = 'opportunities'

    def __unicode__(self):
        return self.description

    @models.permalink
    def get_absolute_url(self):
        return ('opportunity_detail', [self.id])

class ActionLogEntry(models.Model):
    timestamp = models.DateTimeField(_("Timestamp"))
    who = models.ForeignKey(User)
    action = models.TextField(_("Action"), max_length=200)
    opportunity = models.ForeignKey(Opportunity, blank=True)

def log_action(who, action=None, opportunity=None):
    log_entry = ActionLogEntry(timestamp=datetime.datetime.now(),
                               who=who, action=action,
                               opportunity=opportunity)
    log_entry.save()

