# -*- coding: utf-8 -*-
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.utils.translation import ugettext as _
from django.db.models import Count
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response as render
from django.http import HttpResponseRedirect

from django.template import RequestContext
from django.views.generic import list_detail

import models
import forms

from filters import HarvestFilters
from wrappers import PackageWrapper, PackageListWrapper

def opportunity_detail(request, opportunity_id):
    return list_detail.object_detail(
        request,
        queryset = models.Opportunity.objects.filter(last_updated=opportunitylist__last_updated),
        object_id = opportunity_id,
        template_object_name = 'opportunity',
    )

@login_required
def opportunity_edit(request, opportunity_id):
    opportunity = get_object_or_404(models.Opportunity, id=opportunity_id)
    if request.method == "POST":
        form = forms.OpportunityForm(data=request.POST, instance=opportunity)
        if form.is_valid():
            if form.cleaned_data["reviewed"] != opportunity.reviewed:
                models.log_action(request.user, action="marked as reviewed", 
                                  opportunity=opportunity)
            if form.cleaned_data["applied"] != opportunity.applied:
                models.log_action(request.user, action="marked as applied", 
                                  opportunity=opportunity)
            if form.cleaned_data["experience"] != opportunity.experience:
                models.log_action(request.user, 
                                  action="changed experience to: %s" % form.cleaned_data["experience"],
                                  opportunity=opportunity)
            if form.cleaned_data["comment"] != opportunity.comment:
                if len(form.cleaned_data["comment"]) > 178:
                    action = "changed comment to: '%s'" % (form.cleaned_data["comment"][:177]+u"…")
                else:
                    action = "changed comment to: '%s'" % form.cleaned_data["comment"]
                models.log_action(request.user, action=action,
                                  opportunity=opportunity)
            form.save()
            return HttpResponseRedirect(request.POST["next"])
        else:
            request.user.message_set.create(message=_('Opportunity details could not be saved.'))
    else:
        form = forms.OpportunityForm(instance=opportunity)
    return render('opportunities/opportunity_edit.html', 
                    {'form': form, 
                     'opportunity':opportunity,
                     'user':request.user, 
                     'next': request.GET['next'],
                    }, RequestContext(request))


def _create_packages_list(request, filters_pkg, filters_opp):
    # XXX: rockstar: Eep! We shouldn't be storing the None as a string.  We
    # should re-think this model relationship.
    #sourcepackages_list = models.SourcePackage.objects.exclude(name='None')
    
    sourcepackages_list = models.SourcePackage.objects.distinct()
    sourcepackages_list = filters_pkg.process_queryset(sourcepackages_list)
    
    #opportunities_list is filtered right away to only check opportunities belonging to selected packages
    opportunities_list = models.Opportunity.objects.distinct().filter(sourcepackage__in=sourcepackages_list)
    opportunities_list = filters_opp.process_queryset(opportunities_list)
    
    #TODO: need to filter out opportunities with valid=False again
    #TODO: would it be more efficient to group opportunities by their sourcepackages first, then run filters_opp.process_queryset() for each of those groups?
    
    pkg_list_wrapper = PackageListWrapper(request, sourcepackages_list, opportunities_list)
    
    return pkg_list_wrapper


def opportunities_filter(request):
    filters = HarvestFilters()
    filters.update_from_http(request)
    filters_pkg = filters.find('pkg')
    filters_opp = filters.find('opp')
    
    pkg_list_wrapper = _create_packages_list(request, filters_pkg, filters_opp) 
    
    context = {
        'grouping': 'package',
        'packages_list': pkg_list_wrapper,
        'filters_pkg' : filters_pkg,
        'filters_opp' : filters_opp
    }

    return render(
        'opportunities/filter.html',
        context,
        context_instance=RequestContext(request))


def opportunities_filter_results(request):
    filters = HarvestFilters()
    filters.update_from_http(request)
    
    packages_list = _create_packages_list(request, filters.find('pkg'), filters.find('opp')) 
    
    context = {
        'grouping': 'package',
        'packages_list': packages_list
    }

    return render(
        'opportunities/filter_results.html',
        context,
        context_instance=RequestContext(request))


def opportunities_package_details(request, package_id):
    filters = HarvestFilters()
    filters.update_from_http(request)
    
    package = models.SourcePackage.objects.get(id=package_id)
    
    opportunities_list = filters.find('opp').process_queryset(package.opportunity_set).all()
    
    package_wrapper = PackageWrapper(request, package, visible_opportunities = opportunities_list, expanded = True)
    
    context = {
        'grouping': 'package',
        'package': package_wrapper
    }
    
    return render(
        'opportunities/package_details.html',
        context,
        context_instance=RequestContext(request))
